<?php

/**
 * Handles the Order.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'GTW_Order_Handler' ) ) {

	/**
	 * Class.
	 */
	class GTW_Order_Handler {

		/**
		 * Class Initialization.
		 */
		public static function init() {
			// Update order meta.
			add_action( 'woocommerce_checkout_create_order_line_item', array( __CLASS__, 'adjust_order_item' ), 10, 4 );
			// Handles the gift wrapper order note.
			add_action( 'woocommerce_checkout_update_order_meta', array( __CLASS__, 'handles_gift_wrapper_order_note' ), 10 );
			// Hide order item meta key.
			add_action( 'woocommerce_hidden_order_itemmeta', array( __CLASS__, 'hide_order_item_meta_key' ), 10, 2 );
		}

		/**
		 * Adjust order item meta
		 */
		public static function adjust_order_item( $item, $cart_item_key, $values, $order ) {
			if ( ! isset( $values['gtw_gift_wrapper'] ) ) {
				return;
			}

			$rule         = gtw_get_product_rule_by_id( $values['product_id'], $values['gtw_gift_wrapper']['rule_id'] );
			$show_designs = gtw_display_gift_wrapper_designs();
			if ( $show_designs && ! $rule['id'] ) {
				return;
			}

			// Update order item meta.
			$item->add_meta_data( '_gtw_gift_rule_id', $values['gtw_gift_wrapper']['rule_id'] );
			$item->add_meta_data( '_gtw_gift_price', $values['gtw_gift_wrapper']['price'] );
			$item->add_meta_data( '_gtw_gift_mode', $values['gtw_gift_wrapper']['mode'] );

			if ( $show_designs ) {
				$design_value = $rule['name'] . ' (' . gtw_price( gtw_convert_price( $rule['price'] ) ) . ')';
			} else {
				$design_value = gtw_price( gtw_convert_price( gtw_get_default_design_price() ) );
			}

			$item->add_meta_data( get_option( 'gtw_settings_gift_wrapper_design_localization' ), $design_value );

			// Update the fields order item.
			self::fields_order_item( $item, $values['gtw_gift_wrapper'] );
						// Update the order gift wrapper values in order meta data.
						self::update_order_meta_data( $order , $values ) ;
		}

		/**
		 * Update the fields order item.
		 *
		 * @return array
		 * */
		public static function fields_order_item( $item, $values ) {
			// Check if the fields post data is exists.
			if ( ! isset( $values['fields'] ) ) {
				return;
			}

			$custom_field_ids = GTW_Custom_Fields_Handler::get_custom_fields();
			foreach ( $values['fields'] as $field_key => $field_value ) {
				if ( ! isset( $custom_field_ids[ $field_key ] ) ) {
					continue;
				}

				if ( empty( $field_value ) ) {
					continue;
				}

				$item->add_meta_data( $custom_field_ids[ $field_key ]->get_name(), $field_value );
			}

			$item->add_meta_data( '_gtw_gift_fields', $values['fields'] );
		}
				
				/**
		 * Update the order meta data.
		 *
		 * @since 5.3.0
		 * @param array $order 
		 * @param array $values 
		 * @return array
		 * */
		public static function update_order_meta_data( $order, $values ) {
			if ( ! isset( $values[ 'gtw_gift_wrapper' ] ) || 'order' !== $values[ 'gtw_gift_wrapper' ][ 'mode' ] ) {
				return ;
			}
			//Update the gift wrapper rule id if exists.
			if ($values[ 'gtw_gift_wrapper' ][ 'rule_id' ]) {
				$order->update_meta_data( '_gtw_gift_rule_id' , $values[ 'gtw_gift_wrapper' ][ 'rule_id' ] ) ;
			}
			
			$order->update_meta_data( '_gtw_gift_price' , $values[ 'gtw_gift_wrapper' ][ 'price' ] ) ;
			// Check if the fields post data is exists and not empty.
			if ( isset( $values[ 'gtw_gift_wrapper' ][ 'fields' ] ) && !$values[ 'gtw_gift_wrapper' ][ 'fields' ]) {
				$order->update_meta_data( '_gtw_gift_fields' , $values[ 'gtw_gift_wrapper' ][ 'fields' ] ) ;
			}
		}

		/**
		 * Format the custom fields order note.
		 *
		 * @since 4.1
		 * @param string $note
		 * @param array  $fields
		 * @return string
		 * */
		public static function format_custom_fields_order_note( $note, $fields ) {
			if ( ! gtw_check_is_array( $fields ) ) {
				return $note;
			}

			$custom_field_ids = GTW_Custom_Fields_Handler::get_custom_fields();
			foreach ( $fields as $field_key => $field_value ) {
				if ( ! isset( $custom_field_ids[ $field_key ] ) ) {
					continue;
				}

				if ( empty( $field_value ) ) {
					continue;
				}

				$note .= '<p><b>' . $custom_field_ids[ $field_key ]->get_name() . ':</b> ' . $field_value . '</p>';
			}

			return $note;
		}

		/**
		 * Handles the gift wrapper order note.
		 *
		 * @param string $order_id
		 */
		public static function handles_gift_wrapper_order_note( $order_id ) {
			// Return if the gift wrapper order notes are not enabled.
			if ( '2' !== get_option( 'gtw_settings_gift_wrapper_order_notes' ) ) {
				return;
			}

			$order = wc_get_order( $order_id );
			if ( ! is_object( $order ) ) {
				return;
			}

			$notes = array();
			foreach ( $order->get_items() as $item ) {
				// Omit if the product is not a gift wrapper.
				if ( ! isset( $item['gtw_gift_mode'] ) ) {
					continue;
				}

				$rule         = gtw_get_product_rule_by_id( $item['product_id'], $item['gtw_gift_rule_id'] );
				$show_designs = gtw_display_gift_wrapper_designs();
				if ( $show_designs && ! $rule['id'] ) {
					return;
				}

				if ( $show_designs ) {
					$design_value = $rule['name'] . ' (' . gtw_price( gtw_convert_price( $item['gtw_gift_price'] ) ) . ')';
				} else {
					$design_value = gtw_price( gtw_convert_price( $item['gtw_gift_price'] ) );
				}

				if ( 'order' === $item['gtw_gift_mode'] ) {
					$valid_products = gtw_get_order_gift_wrapper_products();
					$note           = '<p><b>' . get_option( 'gtw_settings_gift_wrapper_order_products_localization' ) . ':</b> ' . implode( ', ', $valid_products ) . '</p>';
				} else {
					$note = '<p><b>' . get_option( 'gtw_settings_gift_wrapper_product_name_localization' ) . ':</b> ' . $item->get_product()->get_name() . '</p>';
				}

				$note .= '<p><b>' . get_option( 'gtw_settings_gift_wrapper_design_localization' ) . ':</b> ' . $design_value . '</p>';
				// Custom fields order note.
				$note = self::format_custom_fields_order_note( $note, $item['gtw_gift_fields'] );

				$notes[] = $note;
			}

			// Add notes if gift wrapper exists in the order.
			if ( $notes ) {
				$gift_wrapper_note = '<p><b>' . get_option( 'gtw_settings_gift_wrapper_details_localization' ) . '</b></p></br>';

				foreach ( $notes as $note ) {
					$gift_wrapper_note .= $note . ' </ br>';
				}

				$order->add_order_note( $gift_wrapper_note );
			}
		}

		/**
		 * Hidden the custom order item meta.
		 *
		 * @since 1.0.0
		 * @param array $hidden_order_itemmeta
		 * @return array
		 */
		public static function hide_order_item_meta_key( $hidden_order_itemmeta ) {
			$custom_order_itemmeta = array( '_gtw_gift_rule_id', '_gtw_gift_price', '_gtw_gift_message', '_gtw_gift_mode', '_gtw_gift_fields' );

			return array_merge( $hidden_order_itemmeta, $custom_order_itemmeta );
		}
	}

	GTW_Order_Handler::init();
}
