<?php

/**
 *  Handles the Cart.
 * */
if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}

if (!class_exists('GTW_Cart_Handler')) {

	/**
	 * Class.
	 * */
	class GTW_Cart_Handler {

		/**
		 * Class Initialization.
		 * */
		public static function init() {
			// May be force the product gift wrapper when adding the products.
			add_filter('woocommerce_add_to_cart_validation', array( __CLASS__, 'validate_woocommerce_add_to_cart' ), 10, 3);
			// Add item data to the gift wrapper.
			add_action('woocommerce_add_cart_item_data', array( __CLASS__, 'add_cart_item_data' ), 20, 4);
			// Gets cart item to display in cart.
			add_action('woocommerce_get_item_data', array( __CLASS__, 'display_custom_item_data' ), 10, 2);
			// Alter cart item thumbnail.
			add_action('woocommerce_cart_item_thumbnail', array( __CLASS__, 'alter_cart_item_thumbnail' ), 10, 3);
			// Alter product price based on gift wrapper selection.
			add_filter('woocommerce_get_cart_item_from_session', array( __CLASS__, 'set_price' ), 10, 3);
			// Maybe update the cart item price after gift wrapper product added. 
			add_filter('woocommerce_add_cart_item', array( __CLASS__, 'maybe_update_cart_item_price' ), 20);
			// Update the cart item price based on quantity changed. 
			add_action('woocommerce_after_cart_item_quantity_update', array( __CLASS__, 'maybe_update_price_on_quantity_update' ), 20, 4);
			// Alter order gift wrapper product quantity.
			add_filter('woocommerce_cart_contents_changed', array( __CLASS__, 'set_order_gift_wrapper_quantity' ), 10, 3);
			// set cart quantity in cart page when product is a gift wrapper.
			add_filter('woocommerce_cart_item_quantity', array( __CLASS__, 'set_cart_item_quantity' ), 10, 2);
			// Remove the gift wrapper based on restriction from cart.
			add_action('wp', array( __CLASS__, 'remove_gift_wrapper_from_cart' ));
			// Remove the gift wrapper based on restriction from cart via cart ajax.
			add_action('woocommerce_before_cart', array( __CLASS__, 'remove_gift_wrapper_from_cart_ajax' ));
			// Remove the gift wrapper based on restriction from cart via checkout ajax.
			add_action('woocommerce_review_order_before_cart_contents', array( __CLASS__, 'remove_gift_wrapper_from_cart_ajax' ));
			// Remove order gift wrapper from the cart when the cart is empty.
			add_action('woocommerce_cart_item_removed', array( __CLASS__, 'remove_order_gift_wrapper_cart_empty' ), 10, 2);
			// Edit the gift wrapper from the cart.
			add_action('woocommerce_after_cart_item_name', array( __CLASS__, 'render_edit_gift_wrapper_link' ), 1);
			// Add the product gift wrapper from the cart.
			add_filter('woocommerce_cart_item_name', array( __CLASS__, 'render_product_gift_wrapper_link' ), 10, 2);
			// Alter the cart contents order.
			add_filter('woocommerce_cart_contents_changed', array( __CLASS__, 'alter_cart_contents_order' ), 10, 1);
		}

		/**
		 * May be force the product gift wrapper when adding the products.
		 * 
		 * @return bool
		 * */
		public static function validate_woocommerce_add_to_cart( $bool, $product_id, $qty ) {
			// Return if the product page gift wrapper is not enabled.
			if ('yes' != get_option('gtw_settings_enable_gift_wrapper_product_page')) {
				return $bool;
			}

			// Return if the force product gift wrapper is not enabled.
			if ('2' != get_option('gtw_settings_force_product_gift_wrapper')) {
				return $bool;
			}

			$product = wc_get_product($product_id);
			// Return if the product variable is not object.
			if (!is_object($product)) {
				return $bool;
			}

			// Return if the current product is not valid.
			if (gtw_is_product_gift_wrapper_restricted($product)) {
				return $bool;
			}

			// Return if the gift wrapper is selected.
			if (isset($_REQUEST['gtw_enable_gift_wrapper'])) {
				return $bool;
			}

			gtw_add_notice(__('Please select gift wrapper in order to add this product to cart', 'gift-wrapper-for-woocommerce'), 'error');

			return false;
		}

		/**
		 * Add item data to the gift wrapper.
		 *
		 * @return array
		 * */
		public static function add_cart_item_data( $cart_item_data, $product_id, $variation_id, $quantity = 1 ) {
			$post_data = $_REQUEST;

			//Return if the gift wrapper is not enabled.
			if (!isset($post_data['gtw_enable_gift_wrapper']) || empty($post_data['gtw_enable_gift_wrapper'])) {
				return $cart_item_data;
			}

			// Validate the gift wrapper cart item data.
			/**
			 * This hook is used to validate gift wrapper cart item data.
			 * 
			 * @hooked GTW_Product_Bundles_Compatibility->validate_stamp_products_gift_wrap - 10
			 * @since 1.0
			 */
			if (apply_filters('gtw_validate_gift_wrapper_cart_item_data', false, $cart_item_data, $product_id, $variation_id, $quantity)) {
				return $cart_item_data;
			}

			// Check if the designs are enabled.
			if (gtw_display_gift_wrapper_designs()) {
				//Return if the rule is not valid.
				$rule_id = isset($post_data['gtw_gift_wrapper_item']) ? wc_clean(wp_unslash($post_data['gtw_gift_wrapper_item'])) : '';
				if (!$rule_id) {
					return $cart_item_data;
				}

				//Return if the rule is not exists.
				$rule = gtw_get_product_rule_by_id($product_id, $rule_id);
				if (!$rule['id']) {
					return $cart_item_data;
				}

				$rule_id = $rule['id'];
				$price = $rule['price'];
			} else {
				$rule_id = '';
				$price = get_option('gtw_settings_gift_wrapper_design_price');
			}

			$product_id = !empty($variation_id) ? $variation_id : $product_id;
			$product = wc_get_product($product_id);

			$custom_field_data = array();

			// Check if the fields post data is exists.
			if (isset($post_data['gtw_wrapper_field'])) {
				$custom_field_data = GTW_Custom_Fields_Handler::prepare_fields_cart_item_data($post_data['gtw_wrapper_field']);
			}

			// Prepare the gift wrapper item data.
			$cart_item_data['gtw_gift_wrapper'] = array(
				'rule_id' => $rule_id,
				'product_price' => floatval($product->get_price()) + floatval($price),
				'price' => floatval($price),
				'fields' => $custom_field_data,
				'mode' => 'product',
			);

			return $cart_item_data;
		}

		/**
		 * Gets cart item to display in the cart.
		 *
		 * @return array
		 * */
		public static function display_custom_item_data( $item_data, $cart_item ) {
			if (!isset($cart_item['gtw_gift_wrapper'])) {
				return $item_data;
			}

			$gift_wrapper = $cart_item['gtw_gift_wrapper'];

			//Return if the rule does not exists.
			$rule = gtw_get_product_rule_by_id($cart_item['product_id'], $gift_wrapper['rule_id']);
			$show_designs = gtw_display_gift_wrapper_designs();
			if ($show_designs && !$rule['id']) {
				return $item_data;
			}

			$gift_wrapper_item_data = array();

			$product_id = !empty($cart_item['variation_id']) ? $cart_item['variation_id'] : $cart_item['product_id'];
			$product = wc_get_product($product_id);

			if ($show_designs) {
				$design_value = $rule['name'] . ' (' . gtw_price(gtw_get_price_to_display($product, gtw_convert_price($rule['price']))) . ')';
			} else {
				$design_value = gtw_price(gtw_get_price_to_display($product, gtw_convert_price(gtw_get_default_design_price())));
			}

			// Gift Wrapper Name.
			$gift_wrapper_item_data[] = array(
				'name' => get_option('gtw_settings_gift_wrapper_design_localization'),
				'value' => $gift_wrapper['rule_id'],
				'display' => $design_value . ' ' . self::render_edit_gift_wrapper_link($cart_item),
			);

			// Display the fields value in cart item data.
			if (!empty($gift_wrapper['fields']) && gtw_check_is_array($gift_wrapper['fields'])) {
				$custom_field_ids = GTW_Custom_Fields_Handler::get_custom_fields();
				foreach ($gift_wrapper['fields'] as $field_key => $field_value) {
					if (!isset($custom_field_ids[$field_key])) {
						continue;
					}

					if (empty($field_value)) {
						continue;
					}

					// Gift Wrapper message.
					$gift_wrapper_item_data[] = array(
						'name' => $custom_field_ids[$field_key]->get_name(),
						'display' => $field_value,
					);
				}
			}

			return array_merge($item_data, $gift_wrapper_item_data);
		}

		/**
		 * Alter cart item thumbnail.
		 *
		 * @return mixed
		 * */
		public static function alter_cart_item_thumbnail( $thumbnail, $cart_item, $cart_item_key ) {

			if (!isset($cart_item['gtw_gift_wrapper']['mode'])) {
				return $thumbnail;
			}

			if ('order' != $cart_item['gtw_gift_wrapper']['mode']) {
				return $thumbnail;
			}

			if (!gtw_display_gift_wrapper_designs()) {
				return $thumbnail;
			}

			//Return if the rule does not exists.
			$rule = gtw_get_rule($cart_item['gtw_gift_wrapper']['rule_id']);
			if (!$rule->exists()) {
				return $thumbnail;
			}

			// Gift order image.
			return $rule->get_image();
		}

		/**
		 * Alter product price based on gift wrapper selection.
		 *
		 * @return array.
		 */
		public static function set_price( $session_data, $values, $key ) {
			// Return if the current product is not a gift wrapper.
			if (!isset($session_data['gtw_gift_wrapper'])) {
				return $session_data;
			}

			if (!is_object($session_data['data'])) {
				return $session_data;
			}

			return self::update_cart_item_price($session_data);
		}

		/**
		 * Maybe update the cart item price after gift wrapper product added.
		 *
		 * @since 5.3.0
		 * @param array $cart_item_data Cart item meta data.
		 *
		 * @return array
		 */
		public static function maybe_update_cart_item_price( $cart_item_data ) {
			if (!isset($cart_item_data['gtw_gift_wrapper'])) {
				return $cart_item_data;
			}

			return self::update_cart_item_price($cart_item_data);
		}

		/**
		 * Maybe update cart item price based on quantity changed.
		 *
		 * @since 5.3.0
		 * @param string   $cart_item_key
		 * @param integer  $quantity      
		 * @param integer  $old_quantity  
		 * @param \WC_Cart $cart
		 *
		 * @return array
		 */
		public static function maybe_update_price_on_quantity_update( $cart_item_key, $quantity, $old_quantity, $cart ) {
			$cart_item_data = $cart->get_cart_item($cart_item_key);
			if (!isset($cart_item_data['gtw_gift_wrapper'])) {
				return;
			}

			// Set new cart item data, when cart item quantity changes.
			$contents = $cart->get_cart_contents();
			$contents[$cart_item_key] = self::update_cart_item_price($cart_item_data);
			$cart->set_cart_contents($contents);
		}

		/**
		 * Update the gift wrapper cart item price.
		 * 
		 * @since 5.3.0
		 * @param array $cart_item
		 * @return array
		 */
		public static function update_cart_item_price( $cart_item ) {
			/**
			 * This hook is used to alter the gift wrapper price before set in the cart session.
			 * 
			 * @hooked GTW_WooCommerce_Multi_Currency_Compatibility->convert_cart_gift_wrapper_price - 10
			 * @hooked GTW_WPML_Multi_Currency_Compatibility->convert_cart_gift_wrapper_price - 10
			 * @since 1.0
			 */
			$wrapper_price = apply_filters('gtw_gift_wrapper_product_price', floatval($cart_item['gtw_gift_wrapper']['price']), $cart_item);
			$price = $wrapper_price;
			if ('order' != $cart_item['gtw_gift_wrapper']['mode']) {
				/**
				 * This hook is used to alter the cart product original price before set in the cart session.
				 * 
				 * @hooked GTW_WooCommerce_Multi_Currency_Compatibility->convert_cart_product_original_price - 10
				 * @since 4.6
				 */
				$price = apply_filters('gtw_cart_product_original_price', floatval($cart_item['data']->get_price()), $cart_item['data']) + $price;
			}

			$cart_item['data']->set_price($price);
			$cart_item['data']->add_meta_data('gtw_orginal_wrapper_price', $wrapper_price);

			return $cart_item;
		}

		/**
		 * Alter order gift wrapper product quantity.
		 *
		 * @since 5.3.0
		 * @param array $cart_contents 
		 * @return array.
		 */
		public static function set_order_gift_wrapper_quantity( $cart_contents ) {
			// Return if the cart contents is empty.
			if (!gtw_check_is_array($cart_contents)) {
				return $cart_contents;
			}

			// Return if the cart contents is empty.
			if (!gtw_check_is_array($cart_contents)) {
				return $cart_contents;
			}

			$order_gift_wrapper_cart_contents = array();
			foreach ($cart_contents as $key => $cart_content) {
				// Continue if the current product is not a gift wrapper.
				if (!isset($cart_content['gtw_gift_wrapper'])) {
					continue;
				}

				// Continue if the cart content is not order gift wrapper.
				if ('order' !== $cart_content['gtw_gift_wrapper']['mode']) {
					continue;
				}

				// Add the order gift wrapper quantity.
				$quantity = gtw_order_gift_wrapper_cart_qty();
				/**
				 * This hook is used to alter the order gift wrapper cart item quantity.
				 * 
				 * @since 1.0
				 */
				$cart_content['quantity'] = apply_filters('gtw_order_gift_wrapper_cart_item_quantity', $quantity, $key, $cart_content);

				$order_gift_wrapper_cart_contents[$key] = $cart_content;
				unset($cart_contents[$key]);
			}

			return array_merge($cart_contents, $order_gift_wrapper_cart_contents);
		}

		/**
		 * Set cart quantity as non editable in the cart.
		 *
		 * @return int/string
		 */
		public static function set_cart_item_quantity( $quantity, $cart_item_key ) {
			// Return if a cart object is not initialized.
			if (!is_object(WC()->cart)) {
				return $quantity;
			}

			$cart_items = WC()->cart->get_cart();
			// Check if the product is a gift wrapper product.
			if (!isset($cart_items[$cart_item_key]['gtw_gift_wrapper']['mode'])) {
				return $quantity;
			}

			if ('order' !== $cart_items[$cart_item_key]['gtw_gift_wrapper']['mode']) {
				return $quantity;
			}

			/**
			 * This hook is used to alter the order gift wrapper cart item quantity.
			 * 
			 * @since 1.0.0
			 */
			return apply_filters('gtw_order_gift_wrapper_cart_item_quantity', gtw_order_gift_wrapper_cart_qty(), $cart_item_key);
		}

		/**
		 * Remove the gift wrapper based on restriction from the cart.
		 *
		 * @return void
		 * */
		public static function remove_gift_wrapper_from_cart() {
			if (isset($_REQUEST['payment_method']) || isset($_REQUEST['woocommerce-cart-nonce'])) {
				return;
			}

			self::remove_gift_wrapper();
		}

		/**
		 * Render the edit gift wrapper link.
		 *
		 * @param array $cart_item
		 * @since 5.2.0
		 * @return void
		 * */
		public static function render_edit_gift_wrapper_link( $cart_item ) {
			// Return if the gift wrapper custom fields and designs disable. 
			if (empty(gtw_get_active_custom_field_ids()) && !gtw_display_gift_wrapper_designs()) {
				return;
			}

			return gtw_get_template_html('gift-wrapper-edit-link.php', array( 'cart_item_key' => $cart_item['key'] ));
		}

		/**
		 * Render the product gift wrapper link.
		 *
		 * @param string $product_name
		 * @param array $cart_item
		 * @since 5.3.0
		 * @return void
		 * */
		public static function render_product_gift_wrapper_link( $product_name, $cart_item ) {
			return $product_name . ' ' . self::add_product_gift_wrapper_link($cart_item);
		}

		/**
		 * Add the gift wrapper link.
		 *
		 * @param array $cart_item
		 * @since 5.3.0
		 * @return string
		 * */
		public static function add_product_gift_wrapper_link( $cart_item ) {
			if (isset($cart_item['gtw_gift_wrapper'])) {
				return '';
			}

			if ('yes' !== get_option('gtw_settings_enable_gift_wrapper_product_page')) {
				return '';
			}

			// Return if the product gift wrapper is not valid.
			/**
			 * This hook is used to check if a product gift wrapper is valid to display.
			 * 
			 * @since 5.3.0
			 */
			if (!apply_filters('gtw_is_valid_edit_product_gift_wrapper', true, $cart_item, $cart_item['product_id'], $cart_item['variation_id'])) {
				return '';
			}

			// Return if the current product is not valid.
			if (gtw_is_product_gift_wrapper_restricted(wc_get_product($cart_item['product_id']))) {
				return '';
			}

			// Don't consider the virtual products for product gift wrapper based on selected option.
			if ('yes' === get_option('gtw_settings_exclude_virtual_products') && wc_get_product($cart_item['product_id'])->is_virtual()) {
				return '';
			}

			// Return if the order gift wrapper is in cart and display order product level gift wrapper option disabled.
			if (!gtw_allow_both_gift_wrappers() && gtw_order_gift_wrapper_in_cart()) {
				return '';
			}

			// Return if the gift wrapper custom fields and designs disable. 
			if (empty(gtw_get_active_custom_field_ids()) && !gtw_display_gift_wrapper_designs()) {
				return '';
			}

			return gtw_get_template_html('add-product-gift-wrapper-link.php', array( 'cart_item_key' => $cart_item['key'] ));
		}

		/**
		 * Alter the cart contents order.
		 * 
		 * @since 5.2.0
		 * @param array $cart_contents
		 * @return array
		 * */
		public static function alter_cart_contents_order( $cart_contents ) {
			// Return the same cart content if contents is empty.
			if (empty($cart_contents)) {
				return $cart_contents;
			}

			$gtw_cart_contents = array();
			foreach ($cart_contents as $key => $cart_content) {
				if (!isset($cart_content['gtw_cart_item_position'])) {
					continue;
				}

				$gtw_cart_contents[$cart_content['gtw_cart_item_position']] = $key;
			}

			if (empty($gtw_cart_contents)) {
				return $cart_contents;
			}

			// To preserve cart keys, we are doing here.
			$current_position = 0;
			$altered_cart_contents = array();

			foreach ($cart_contents as $key => $cart_content) {
				if (isset($altered_cart_contents[$key])) {
					continue;
				}

				if (isset($gtw_cart_contents[$current_position])) {
					$cart_content = $cart_contents[$gtw_cart_contents[$current_position]];
					unset($cart_content['gtw_cart_item_position']);

					$altered_cart_contents[$gtw_cart_contents[$current_position]] = $cart_content;
				} else {
					$altered_cart_contents[$key] = $cart_content;
				}

				$current_position++;
			}

			return $altered_cart_contents;
		}

		/**
		 * Remove the gift wrapper based on restriction from the cart via ajax.
		 *
		 * @return void
		 * */
		public static function remove_gift_wrapper_from_cart_ajax() {
			if (!isset($_REQUEST['payment_method']) && !isset($_REQUEST['woocommerce-cart-nonce'])) {
				return;
			}

			self::remove_gift_wrapper();
		}

		/**
		 * Remove the gift wrapper based on restriction.
		 *
		 * @return void
		 * */
		public static function remove_gift_wrapper() {
			$order_gift_wrapper_cart_key = gtw_order_gift_wrapper_in_cart();

			if ($order_gift_wrapper_cart_key) {
				self::remove_order_gift_wrapper($order_gift_wrapper_cart_key);
			} else {
				self::remove_product_gift_wrapper();
			}
		}

		/**
		 * Remove the order gift wrapper based on restriction.
		 *
		 * @return void
		 * */
		public static function remove_order_gift_wrapper( $cart_item_key ) {
			// Return if the cart object is not initialized.
			if (!is_object(WC()->cart)) {
				return;
			}

			// Return if the cart contains valid order gift wrapper.
			if (gtw_get_order_gift_wrapper_products('id')) {
				return;
			}

			// Remove order gift wrapper.
			WC()->cart->remove_cart_item($cart_item_key);

			// Error Notice.
			gtw_add_notice(get_option('gtw_settings_order_gift_wrapper_not_eligible_notice_localization'), 'notice');
		}

		/**
		 * Remove the product gift wrapper based on restriction.
		 *
		 * @return void
		 * */
		public static function remove_product_gift_wrapper() {
			// Return if the cart object is not initialized.
			if (!is_object(WC()->cart)) {
				return;
			}

			$products_removed = false;

			foreach (WC()->cart->get_cart() as $key => $value) {
				if (!isset($value['gtw_gift_wrapper'])) {
					continue;
				}

				if (!gtw_is_product_gift_wrapper_restricted($value['data'])) {
					continue;
				}

				// Remove the gift wrapper from the cart.
				WC()->cart->remove_cart_item($key);

				$products_removed = true;
			}

			// Error Notice.
			if ($products_removed) {
				gtw_add_notice(get_option('gtw_settings_order_gift_wrapper_not_eligible_notice_localization'), 'notice');
			}
		}

		/**
		 * Remove order gift wrapper from the cart when a cart is empty.
		 *
		 * @return void
		 * */
		public static function remove_order_gift_wrapper_cart_empty( $removed_cart_item_key, $cart ) {
			// Return if the cart object is not initialized.
			if (!is_object(WC()->cart)) {
				return;
			}

			// Return if the cart is empty.
			if (WC()->cart->get_cart_contents_count() == 0) {
				return;
			}

			$order_gift_wrapper_count = gtw_get_order_gift_wrapper_cart_count();
			$cart_items_count = WC()->cart->get_cart_contents_count() - $order_gift_wrapper_count;

			// Return if products are exists.
			if ($cart_items_count) {
				return;
			}

			// Remove all gift wrappers from the cart.
			WC()->cart->empty_cart();

			// Notice.
			gtw_add_notice(get_option('gtw_settings_order_gift_wrapper_not_eligible_notice_localization'), 'notice');
		}
	}

	GTW_Cart_Handler::init();
}
