<?php

/**
 *  Handles the frontend.
 * */
if ( ! defined( 'ABSPATH' ) ) {
	exit ; // Exit if accessed directly.
}

if ( ! class_exists( 'GTW_Frontend' ) ) {

	/**
	 * Class.
	 * */
	class GTW_Frontend {

		/**
		 * Class Initialization.
		 * */
		public static function init() {

			// May be alter product add to cart label on the shop page.
			add_filter( 'woocommerce_product_add_to_cart_text' , array( __CLASS__, 'maybe_alter_product_add_to_cart_label' ) , 999 , 2 ) ;
			// May be alter product add to cart url on the shop page.
			add_filter( 'woocommerce_loop_add_to_cart_link' , array( __CLASS__, 'maybe_alter_product_add_to_cart_url' ) , 999 , 3 ) ;
			// Define the order gift wrapper hooks.
			add_action( 'wp_head' , array( __CLASS__, 'define_order_gift_wrapper_hooks' ) ) ;
			// Render the gift wrapper popup for the single product page.
			add_action( 'woocommerce_before_add_to_cart_button' , array( __CLASS__, 'render_popup_product_gift_wrapper' ) ) ;
			// Render the gift wrapper for the simple product.
			add_action( 'woocommerce_before_add_to_cart_button' , array( __CLASS__, 'render_gift_wrapper_simple_product' ) ) ;
			// Alter the array of data for a variation. Used in the add to cart form.
			add_action( 'woocommerce_available_variation' , array( __CLASS__, 'render_gift_wrapper_variable_product' ) , 10 , 3 ) ;
			// Render the product gift wrapper items.
			add_action( 'gtw_product_gift_wrapper_items' , array( __CLASS__, 'render_product_gift_wrapper_items' ) , 10 , 1 ) ;
			// Render the product slide gift wrapper items.
			add_action( 'gtw_product_gift_wrapper_items' , array( __CLASS__, 'render_slide_product_gift_wrapper_items' ) , 10 , 1 ) ;
			// Render product gift wrapper fields.
			add_action( 'gtw_after_product_gift_wrapper_summary' , array( __CLASS__, 'render_product_gift_wrapper_custom_fields' ) , 20 ) ;
			// Render order gift wrapper fields.
			add_action( 'gtw_after_order_gift_wrapper_summary' , array( __CLASS__, 'render_order_gift_wrapper_custom_fields' ) , 1 ) ;
			// Remove order gift wrapper notice for the page.
			add_action( 'wp_head' , array( __CLASS__, 'add_remove_order_gift_wrapper_notice' ) ) ;
			// Order gift wrapper exclude products notice for the page.
			add_action( 'wp_head' , array( __CLASS__, 'order_gift_wrapper_exclude_products_notice' ) ) ;
		}

		/**
		 * May be alter product add to cart label on the shop page.
		 * 
		 * @param string $label 
		 * @param object $product
		 * @since 4.0
		 * 
		 * @return string
		 */
		public static function maybe_alter_product_add_to_cart_label( $label, $product ) {
			// Return, if the product variable is not object.
			if ( ! is_object( $product ) ) {
				return $label ;
			}

			// Return If the product is a variable product type.
			if ( $product->is_type( 'variable' ) ) {
				return $label ;
			}

			// Return if the product page gift wrapper is not enabled.
			if ( 'yes' != get_option( 'gtw_settings_enable_gift_wrapper_product_page' ) ) {
				return $label ;
			}

			// Return if the force product gift wrapper is not enabled.
			if ( '2' != get_option( 'gtw_settings_force_product_gift_wrapper' ) ) {
				return $label ;
			}

			// Return if the order gift wrapper is in cart.
			if ( gtw_order_gift_wrapper_in_cart() ) {
				return $label ;
			}

			// Return if the current product is not valid.
			if ( gtw_is_product_gift_wrapper_restricted( $product ) ) {
				return $label ;
			}

			return get_option( 'gtw_settings_product_gift_wrapper_shop_add_to_cart_label' ) ;
		}

		/**
		 * May be alter product add to cart URL on the shop page.
		 * 
		 * @param string $url 
		 * @param object $product
		 * @since 4.0
		 * 
		 * @return string
		 */
		public static function maybe_alter_product_add_to_cart_url( $url, $product, $args ) {
			// Return, if the product variable is not object.
			if ( ! is_object( $product ) ) {
				return $url ;
			}

			// Return If the product is a variable product type.
			if ( $product->is_type( 'variable' ) ) {
				return $url ;
			}

			// Return if the product page gift wrapper is not enabled.
			if ( 'yes' != get_option( 'gtw_settings_enable_gift_wrapper_product_page' ) ) {
				return $url ;
			}

			// Return if the force product gift wrapper is not enabled.
			if ( '2' != get_option( 'gtw_settings_force_product_gift_wrapper' ) ) {
				return $url ;
			}

			// Return if the order gift wrapper is in cart.
			if ( gtw_order_gift_wrapper_in_cart() ) {
				return $url ;
			}

			// Return if the current product is not valid.
			if ( gtw_is_product_gift_wrapper_restricted( $product ) ) {
				return $url ;
			}

			return sprintf( '<a href="%s" class="%s">%s</a>' , $product->get_permalink() , 'button' , get_option( 'gtw_settings_product_gift_wrapper_shop_add_to_cart_label' ) ) ;
		}

		/**
		 * Define the order gift wrapper hook.
		 *
		 * @return void
		 * */
		public static function define_order_gift_wrapper_hooks() {
			// Hook for the cart order gift wrapper.
			$cart_location = self::get_cart_current_location() ;
			if ( gtw_check_is_array( $cart_location ) ) {
				add_action( $cart_location[ 'hook' ] , array( __CLASS__, 'render_order_gift_wrapper_cart' ) , $cart_location[ 'priority' ] ) ;
			}

			// Hook for the checkout order gift wrapper.
			$checkout_location = self::get_checkout_current_location() ;
			if ( gtw_check_is_array( $checkout_location ) ) {
				add_action( $checkout_location[ 'hook' ] , array( __CLASS__, 'render_order_gift_wrapper_checkout' ) , $checkout_location[ 'priority' ] ) ;
			}
		}

		/**
		 * Get the cart gift wrapper current location.
		 *
		 * @return array.
		 */
		public static function get_cart_current_location() {
			$cart_location = get_option( 'gtw_settings_cart_order_gift_wrapper_location' ) ;

			/**
			 * This hook is used to alter order gift wrapper cart locations.
			 * 
			 * @since 1.0
			 */
			$location_details = apply_filters( 'gtw_order_gift_wrapper_cart_locations' , array(
				'1' => array(
					'hook'     => 'woocommerce_before_cart_table',
					'priority' => 9,
				),
				'2' => array(
					'hook'     => 'woocommerce_after_cart_table',
					'priority' => 10,
				),
			) ) ;

			$location_detail = isset( $location_details[ $cart_location ] ) ? $location_details[ $cart_location ] : reset( $location_details ) ;

			/**
			 * This hook is used to alter order gift wrapper cart current location.
			 * 
			 * @since 1.0
			 */
			return apply_filters( 'gtw_order_gift_wrapper_cart_current_location' , $location_detail , $cart_location , $location_details ) ;
		}

		/**
		 * Get the checkout gift wrapper current location.
		 *
		 * @return array.
		 */
		public static function get_checkout_current_location() {
			$checkout_location = get_option( 'gtw_settings_checkout_order_gift_wrapper_location' ) ;

			/**
			 * This hook is used to alter order gift wrapper checkout locations.
			 * 
			 * @since 1.0
			 */
			$location_details = apply_filters( 'gtw_order_gift_wrapper_checkout_locations' , array(
				'1' => array(
					'hook'     => 'woocommerce_before_checkout_form',
					'priority' => 20,
				),
				'2' => array(
					'hook'     => 'woocommerce_checkout_before_order_review_heading',
					'priority' => 10,
				),
				'3' => array(
					'hook'     => 'woocommerce_before_checkout_billing_form',
					'priority' => 10,
				),
				'4' => array(
					'hook'     => 'woocommerce_after_checkout_billing_form',
					'priority' => 10,
				),
				'5' => array(
					'hook'     => get_option( 'gtw_settings_order_gift_wrapper_custom_hook_name' ),
					'priority' => get_option( 'gtw_settings_order_gift_wrapper_custom_hook_priority' ),
				),
			) ) ;

			$location_detail = isset( $location_details[ $checkout_location ] ) ? $location_details[ $checkout_location ] : reset( $location_details ) ;

			/**
			 * This hook is used to alter order gift wrapper checkout current location.
			 * 
			 * @since 1.0
			 */
			return apply_filters( 'gtw_order_gift_wrapper_checkout_current_location' , $location_detail , $checkout_location , $location_details ) ;
		}

		/**
		 * Render gift wrapper for the simple product.
		 *
		 * @since 5.3.0
		 * @return void
		 * */
		public static function render_gift_wrapper_simple_product() {
			global $product ;

			// Return null, if the product variable is not object.
			if ( ! is_object( $product ) ) {
				return null ;
			}

			// Return If the product is a variable product type.
			// Variable product gift card display type is not 2.
			if ( $product->is_type( 'variable' ) && '2' !== get_post_meta( $product->get_id() , '_gtw_variable_product_display_type' , true ) ) {
				return ;
			}

			// Return if the order gift wrapper is in cart and display order product level gift wrapper option disabled.
			if ( !gtw_allow_both_gift_wrappers() && gtw_order_gift_wrapper_in_cart() ) {
				return ;
			}

			// Render gift wrapper.
			self::render_gift_wrapper( $product , $product->get_id() ) ;
		}

		/**
		 * Render gift wrapper for the variable product.
		 *
		 * @since 5.3.0
		 * @return array
		 * 
		 * */
		public static function render_gift_wrapper_variable_product( $array, $class, $variation ) {
			// Return if the array is empty.
			if ( ! gtw_check_is_array( $array ) ) {
				return $array ;
			}

			// Return if the variation variable is not object.
			if ( ! is_object( $variation ) ) {
				return $array ;
			}

			// Return if variable product gift card display type is not 2.
			if ( '2' === get_post_meta( $variation->get_parent_id() , '_gtw_variable_product_display_type' , true ) ) {
				return $array ;
			}

			// Return if the display order product level gift wrapper option disabled and order gift wrapper is in cart.
			if ( !gtw_allow_both_gift_wrappers() && gtw_order_gift_wrapper_in_cart() ) {
				return $array ;
			}

			// Get gift wrapper html.
			$gift_wrapper_html = self::render_gift_wrapper( $variation , $variation->get_parent_id() , $variation->get_id() , false ) ;

			if ( ! $gift_wrapper_html ) {
				return $array ;
			}

			$array[ 'gtw_gift_wrapper' ] = $gift_wrapper_html ;

			return $array ;
		}

		/**
		 * Render Gift Wrapper.
		 *
		 * @return mixed
		 * */
		public static function render_gift_wrapper( $product, $product_id, $variation_id = false, $echo = true ) {

			// Return null, if the product variable is not object.
			if ( ! is_object( $product ) ) {
				return null ;
			}

			// Return if the product page gift wrapper is not enabled.
			if ( 'yes' != get_option( 'gtw_settings_enable_gift_wrapper_product_page' ) ) {
				return null ;
			}

			// Return if the product gift wrapper is not valid.
			/**
			 * This hook is used to check if a product gift wrapper is valid to display.
			 * 
			 * @since 1.0
			 */
			if ( ! apply_filters( 'gtw_is_valid_product_gift_wrapper' , true , $product_id , $variation_id ) ) {
				return null ;
			}

			// Return if the current product is not valid.
			if ( gtw_is_product_gift_wrapper_restricted( $product ) ) {
				return null ;
			}

			// Don't consider the virtual products for product gift wrapper based on selected option.
			if ( 'yes' === get_option( 'gtw_settings_exclude_virtual_products' ) && $product->is_virtual() ) {
				return null ;
			}

			if ( gtw_display_gift_wrapper_designs() ) {
				$gift_wrappers = gtw_get_product_gift_wrapper_rules( $product_id ) ;
				// Return if the rules are not configured.
				if ( ! gtw_check_is_array( $gift_wrappers ) ) {
					return null ;
				}

				$current_gift_wrapper = reset( $gift_wrappers ) ;
				$current_rule         = gtw_format_product_rule( $current_gift_wrapper ) ;
				$rule_price           = gtw_convert_price( $current_rule[ 'price' ] ) ;
				$total_payable        = gtw_get_total_payable_price( $product , $rule_price ) ;

				$data_args = array(
					'product'            => $product,
					'total_payable'      => gtw_price( $total_payable ),
					'gift_wrapper_name'  => $current_rule[ 'name' ],
					'gift_wrapper_price' => gtw_price( gtw_get_price_to_display( $product , $rule_price ) ),
				) ;
			} else {
				$total_payable = gtw_get_total_payable_price( $product , gtw_convert_price( gtw_get_default_design_price() ) ) ;

				$data_args = array(
					'product'       => $product,
					'total_payable' => gtw_price( $total_payable ),
				) ;
			}

			if ( $echo ) {
				// Display gift wrapper.
				gtw_get_template( 'product-gift-wrapper.php' , $data_args ) ;
			} else {
				// Get gift wrapper html.
				return gtw_get_template_html( 'product-gift-wrapper.php' , $data_args ) ;
			}
		}

		/**
		 * Render the product gift wrapper items for the single product page.
		 *
		 * @return void
		 * */
		public static function render_product_gift_wrapper_items( $product ) {
			if ( ! is_object( $product ) ) {
				return ;
			}

			// Return if the display type is not popup.
			if ( 'slide' === gtw_get_product_gift_wrapper_display_type() ) {
				return ;
			}

			// Retun if the designs are not enabled.
			if ( ! gtw_display_gift_wrapper_designs() ) {
				return ;
			}

			$product_id    = ( $product->get_parent_id() ) ? $product->get_parent_id() : $product->get_id() ;
			$gift_wrappers = gtw_get_product_gift_wrapper_rules( $product_id ) ;
			// Return if the rules are not configured.
			if ( ! gtw_check_is_array( $gift_wrappers ) ) {
				return ;
			}

			$see_more = false ;
			if ( count( $gift_wrappers ) > 4 ) {
				$gift_wrappers = array_slice( $gift_wrappers , 0 , 3 ) ;

				$see_more = true ;
			}

			$design_type  = get_post_meta( $product_id , '_gtw_design_type' , true ) ;
			$current_item = ( '2' == $design_type ) ? key( $gift_wrappers ) : reset( $gift_wrappers ) ;

			$data_args = array(
				'gift_wrappers' => $gift_wrappers,
				'see_more'      => $see_more,
				'current_item'  => $current_item,
			) ;

			// Display the product gift wrapper items.
			gtw_get_template( 'product-gift-wrapper-items.php' , $data_args ) ;
		}

		/**
		 * Render the slide gift wrapper items for the single product page.
		 *
		 * @return void
		 * */
		public static function render_slide_product_gift_wrapper_items( $product ) {
			if ( ! is_object( $product ) ) {
				return ;
			}

			// Return if the display type is not popup.
			if ( 'slide' !== gtw_get_product_gift_wrapper_display_type() ) {
				return ;
			}

			// Retun if the designs are not enabled.
			if ( ! gtw_display_gift_wrapper_designs() ) {
				return ;
			}

			$product_id    = ( $product->get_parent_id() ) ? $product->get_parent_id() : $product->get_id() ;
			$gift_wrappers = gtw_get_product_gift_wrapper_rules( $product_id ) ;
			// Return if the rules are not configured.
			if ( ! gtw_check_is_array( $gift_wrappers ) ) {
				return ;
			}

			$current_page = 1 ;
			$design_type  = get_post_meta( $product_id , '_gtw_design_type' , true ) ;
			$current_item = ( '2' == $design_type ) ? key( $gift_wrappers ) : reset( $gift_wrappers ) ;
			$per_page     = gtw_get_slide_gift_wrapper_per_page_count() ;

			/* Calculate Page Count */
			$default_args[ 'posts_per_page' ] = $per_page ;
			$default_args[ 'offset' ]         = ( $current_page - 1 ) * $per_page ;
			$page_count                     = ceil( count( $gift_wrappers ) / $per_page ) ;

			$data_args = array(
				'gift_wrappers' => array_slice( $gift_wrappers , $default_args[ 'offset' ] , $per_page ),
				'current_item'  => $current_item,
				'pagination'    => array(
					'page_count'   => $page_count,
					'current_page' => $current_page,
				),
			) ;

			// Display product gift wrappers slide.
			gtw_get_template( 'slide/product-gift-wrapper.php' , $data_args ) ;
		}

		/**
		 * Render product gift wrapper custom fields.
		 *
		 * @return void
		 * */
		public static function render_product_gift_wrapper_custom_fields() {
			// If no one fields are not exists/active.
			$custom_field_ids = GTW_Custom_Fields_Handler::get_custom_fields() ;
			if ( ! gtw_check_is_array( $custom_field_ids ) ) {
				return ;
			}

			// Display gift wrapper fields.
			gtw_get_template( 'product-gift-wrapper-fields.php' , array( 'fields' => $custom_field_ids ) ) ;
		}

		/**
		 * Render gift wrapper popup for the single product page.
		 *
		 * @since 5.3.0
		 * @return void
		 * */
		public static function render_popup_product_gift_wrapper() {
			// Return if the product gift wrapper is not enabled.
			if ( 'yes' != get_option( 'gtw_settings_enable_gift_wrapper_product_page' ) ) {
				return ;
			}

			// Return if the designs is not enabled.
			if ( ! gtw_display_gift_wrapper_designs() ) {
				return ;
			}

			// Return if the display type is not popup.
			if ( 'popup' !== gtw_get_product_gift_wrapper_display_type() ) {
				return ;
			}

			// Return if the display order product level gift wrapper option disabled and order gift wrapper is in cart.
			if ( !gtw_allow_both_gift_wrappers() && gtw_order_gift_wrapper_in_cart() ) {
				return ;
			}

			global $product ;
			$gift_wrappers = gtw_get_product_gift_wrapper_rules( $product->get_id() ) ;
			// Return if the rules are not configured.
			if ( ! gtw_check_is_array( $gift_wrappers ) ) {
				return ;
			}

			$current_page         = 1 ;
			$design_type          = get_post_meta( $product->get_id() , '_gtw_design_type' , true ) ;
			$current_item         = ( '2' == $design_type ) ? key( $gift_wrappers ) : reset( $gift_wrappers ) ;
			$current_gift_wrapper = reset( $gift_wrappers ) ;
			$per_page             = gtw_get_popup_gift_wrapper_per_page_count() ;

			/* Calculate Page Count */
			$default_args[ 'posts_per_page' ] = $per_page ;
			$default_args[ 'offset' ]         = ( $current_page - 1 ) * $per_page ;
			$page_count                     = ceil( count( $gift_wrappers ) / $per_page ) ;

			$current_rule = gtw_format_product_rule( $current_gift_wrapper ) ;

			$data_args = array(
				'product_id'         => $product->get_id(),
				'gift_wrappers'      => array_slice( $gift_wrappers , $default_args[ 'offset' ] , $per_page ),
				'current_item'       => $current_item,
				'gift_wrapper_name'  => $current_rule[ 'name' ],
				'gift_wrapper_price' => gtw_price( gtw_get_price_to_display( $product , gtw_convert_price( $current_rule[ 'price' ] ) ) ),
				'pagination'         => array(
					'page_count'   => $page_count,
					'current_page' => $current_page,
				),
			) ;

			// Display product gift wrapper popup.
			gtw_get_template( 'popup/product-gift-wrapper.php' , $data_args ) ;
		}

		/**
		 * Render gift wrapper for the order in the cart.
		 *
		 * @return void
		 * */
		public static function render_order_gift_wrapper_cart() {
			// Return if the order gift wrapper is not enabled.
			if ( 'yes' != get_option( 'gtw_settings_enable_cart_order_gift_wrapper' ) ) {
				return ;
			}

			// Return if the cart order gift wrapper is not valid.
			/**
			 * This hook is used to check if an order gift wrapper is valid to display in the cart.
			 * 
			 * @since 1.0
			 */
			if ( ! apply_filters( 'gtw_is_valid_cart_order_gift_wrapper' , true ) ) {
				return ;
			}

			self::render_order_gift_wrapper( 'cart' ) ;
		}

		/**
		 * Render gift wrapper for the order in the checkout.
		 *
		 * @return void
		 * */
		public static function render_order_gift_wrapper_checkout() {
			// Return if the order gift wrapper is not enabled.
			if ( 'yes' != get_option( 'gtw_settings_enable_checkout_order_gift_wrapper' ) ) {
				return ;
			}

			// Return if the checkout order gift wrapper is not valid.
			/**
			 * This hook is used to check if an order gift wrapper is valid to display in the checkout.
			 * 
			 * @since 1.0
			 */
			if ( ! apply_filters( 'gtw_is_valid_checkout_order_gift_wrapper' , true ) ) {
				return ;
			}

			self::render_order_gift_wrapper( 'checkout' ) ;
		}

		/**
		 * Render gift wrapper for the order.
		 *
		 * @since 5.3.0
		 * @return void
		 * */
		public static function render_order_gift_wrapper( $page ) {

			//Return if the order gift wrapper product is not configured.
			$product_id = gtw_get_order_gift_wrapper_product() ;
			if ( empty( $product_id ) ) {
				return ;
			}

			// Return if the display order product level gift wrapper option disabled and gift wrapper is in the cart.
			if ( !gtw_allow_both_gift_wrappers() && gtw_product_gift_wrapper_in_cart() ) {
				return ;
			}

			// Return if the gift wrapper is in the cart and gift wrapper button field type.
			if ( gtw_order_gift_wrapper_in_cart() && '1' == gtw_get_order_gift_wrapper_field_type() ) {
				return ;
			}

			// Return if the order gift wrapper is not valid.
			/**
			 * This hook is used to check if an order gift wrapper is valid to display.
			 * 
			 * @since 1.0
			 */
			if ( ! apply_filters( 'gtw_is_valid_order_gift_wrapper' , true ) ) {
				return ;
			}

			// Return if the display order product level gift wrapper option disabled and cart is not eligible for order gift wrapper.
			if ( !gtw_allow_both_gift_wrappers() && ! gtw_get_order_gift_wrapper_products( 'id' ) ) {
				return ;
			}

			// Return if the cart contains the exclude products and gift wrapper hide option is enabled.
			if ( gtw_get_order_gift_wrapper_exclude_products() && '3' !== get_option( 'gtw_settings_exclude_order_gift_wrapper_message_type' ) ) {
				return ;
			}

			$extra_content_type = false ;
			$custom_field_ids   = GTW_Custom_Fields_Handler::get_custom_fields() ;
			$product            = wc_get_product( $product_id ) ;
			if ( gtw_display_gift_wrapper_designs() ) {
				// Return if the rules are not configured.
				$gift_wrapper_ids = gtw_get_active_rule_ids() ;
				if ( ! gtw_check_is_array( $gift_wrapper_ids ) ) {
					return ;
				}

				$extra_content_type = gtw_get_order_gift_wrapper_display_type() ;
				$per_page           = ( 'slide' == $extra_content_type ) ? gtw_get_slide_gift_wrapper_per_page_count() : gtw_get_popup_gift_wrapper_per_page_count() ;
				$current_page       = 1 ;

				/* Calculate page count */
				$default_args[ 'posts_per_page' ] = $per_page ;
				$default_args[ 'offset' ]         = ( $current_page - 1 ) * $per_page ;
				$page_count                     = ceil( count( $gift_wrapper_ids ) / $per_page ) ;

				$current_item = reset( $gift_wrapper_ids ) ;
				$curent_rule  = gtw_get_rule( $current_item ) ;

				$data_args = array(
					'product'            => $product,
					'show_designs'       => true,
					'gift_wrapper_ids'   => array_slice( $gift_wrapper_ids , $default_args[ 'offset' ] , $per_page ),
					'current_item'       => $current_item,
					'gift_wrapper_name'  => $curent_rule->get_name(),
					'gift_wrapper_price' => gtw_price( gtw_get_price_to_display( $product , gtw_convert_price( $curent_rule->get_price() ) ) ),
					'pagination'         => array(
						'page_count'   => $page_count,
						'current_page' => $current_page,
					),
				) ;
			} elseif ( gtw_check_is_array( $custom_field_ids ) ) {
				$data_args = array(
					'show_designs' => false,
					'product'      => $product,
				) ;

				$extra_content_type = gtw_get_order_gift_wrapper_display_type() ;
			}


			/**
			 * This hook is used to alter the order gift wrapper button file name.
			 * 
			 * @since 1.0
			 */
			$button_file_name = apply_filters( 'gtw_order_gift_wrapper_button_file_name_' . $page , 'add-order-gift-wrapper.php' ) ;
			// Display the add order gift wrapper buttton.
			gtw_get_template( $button_file_name , array( 'extra_content_type' => $extra_content_type, 'product' => $product ) ) ;

			// Return if the gift wrapper is in the cart.
			if ( gtw_order_gift_wrapper_in_cart() ) {
				return ;
			}
			switch ( $extra_content_type ) {
				case 'popup':
					/**
					 * This hook is used to alter the popup order gift wrapper button file name.
					 * 
					 * @since 1.0
					 */
					$popup_file_name = apply_filters( 'gtw_popup_order_gift_wrapper_file_name_' . $page , 'popup/order-gift-wrapper.php' , $data_args ) ;
					// Display the popup order gift wrapper.
					gtw_get_template( $popup_file_name , $data_args ) ;
					break ;

				case 'slide':
					/**
					 * This hook is used to alter the slide order gift wrapper button file name.
					 * 
					 * @since 1.0
					 */
					$slide_file_name = apply_filters( 'gtw_slide_order_gift_wrapper_file_name_' . $page , 'slide/order-gift-wrapper.php' , $data_args ) ;
					// Display the slide order gift wrapper.
					gtw_get_template( $slide_file_name , $data_args ) ;
					break ;
			}
		}

		/**
		 * Render the order gift wrapper custom fields.
		 *
		 * @param array $prefilled_values
		 * @since 5.2.0
		 * @return array
		 * */
		public static function render_order_gift_wrapper_custom_fields( $prefilled_values ) {
			// If no one fields are not exists/active.
			$custom_field_ids = GTW_Custom_Fields_Handler::get_custom_fields() ;
			if ( ! gtw_check_is_array( $custom_field_ids ) ) {
				return ;
			}

			// Display gift wrapper fields.
			gtw_get_template( 'popup/product-gift-wrapper-fields.php' , array( 'fields' => $custom_field_ids, 'prefilled_values' => $prefilled_values ) ) ;
		}

		/**
		 * Order gift wrapper remove notice for the page.
		 *
		 * @since 5.3.0
		 * @return void
		 * */
		public static function add_remove_order_gift_wrapper_notice() {
			// Return if the order gift wrapper remove notice is disabled.
			if ( '2' == get_option( 'gtw_settings_show_order_gift_wrapper_remove_notice' ) ) {
				return ;
			}

			// Return if the page is not cart/checkout.
			if ( '2' == get_option( 'gtw_settings_order_gw_remove_notice_display_type' ) && ! is_cart() && ! is_checkout() ) {
				return ;
			}

			// Return if the order gift wrapper is not in cart.
			if ( ! gtw_order_gift_wrapper_in_cart() ) {
				return ;
			}

			//Return if the display order product level gift wrapper option enabled.
			if ( gtw_allow_both_gift_wrappers() ) {
				return ;
			}

			if ( ( is_cart() || is_checkout() ) && ( '2' == gtw_get_order_gift_wrapper_field_type() ) ) {
				return ;
			}

			// Get remove order Gift wrapper html.
			$remove_gift_wrapper_html = gtw_get_template_html( 'remove-order-gift-wrapper.php' ) ;

			$remove_wrapper_notice = str_replace( '{remove_link}' , $remove_gift_wrapper_html , get_option( 'gtw_settings_order_gift_wrapper_remove_notice_localization' ) ) ;

			// Add remove order gift wrapper notice.
			gtw_add_notice( $remove_wrapper_notice , 'notice' ) ;
		}

		/**
		 * Order gift wrapper exclude products notice for the page.
		 *
		 * @since 5.3.0
		 * @return void
		 * */
		public static function order_gift_wrapper_exclude_products_notice() {

			// Return If the current page is not checkout/cart.
			if ( ! is_cart() && ! is_checkout() ) {
				return ;
			}

			// Return if the product gift wrapper is in cart and display order product level gift wrapper option disabled.
			if ( gtw_product_gift_wrapper_in_cart() && !gtw_allow_both_gift_wrappers()) {
				return ;
			}

			$exclude_products = gtw_get_order_gift_wrapper_exclude_products() ;
			// Return if the exclude products are exists.
			// Hide exclude products notice option is enabled.
			if ( ! gtw_check_is_array( $exclude_products ) || '1' === get_option( 'gtw_settings_exclude_order_gift_wrapper_message_type' ) ) {
				return ;
			}

			$per_page     = 5 ;
			$current_page = 1 ;

			/* Calculate page count */
			$default_args[ 'posts_per_page' ] = $per_page ;
			$default_args[ 'offset' ]         = ( $current_page - 1 ) * $per_page ;
			$page_count                     = ceil( count( $exclude_products ) / $per_page ) ;

			$data_args = array(
				'exclude_products' => array_slice( $exclude_products , $default_args[ 'offset' ] , $per_page ),
				'pagination'       => array(
					'page_count'   => $page_count,
					'current_page' => $current_page,
				),
			) ;

			// Get html exclude products layout content.
			$html = gtw_get_template_html( 'popup/exclude-products-layout.php' , $data_args ) ;

			$non_applicable_products = $html . "<a href='#' class='gtw-popup-order-exclude-products gtw-popup-gift-wrapper' data-popup='#gtw-popup-order-exclude-items-modal'>" . get_option( 'gtw_settings_exclude_order_gift_wrapper_click_here_label' ) . '</a>' ;

			$exclude_products_notice = str_replace( '{non_applicable_products}' , $non_applicable_products , get_option( 'gtw_settings_exclude_order_gift_wrapper_message' ) ) ;

			// Order gift wrapper exclude products notice.
			gtw_add_notice( $exclude_products_notice , 'notice' ) ;
		}
	}

	GTW_Frontend::init() ;
}
