<?php

/**
 * Frontend functions.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit ; // Exit if accessed directly.
}

if ( ! function_exists( 'gtw_get_order_gift_wrapper_cart_count' ) ) {

	/**
	 * Get order gift wrapper count in cart.
	 *
	 * @return bool/String
	 */
	function gtw_get_order_gift_wrapper_cart_count() {
		$count = 0 ;
		// Return if a cart object is not initialized.
		if ( ! is_object( WC()->cart ) ) {
			return $count ;
		}

		// Return 0 if the cart is empty.
		$cart_contents = WC()->cart->get_cart() ;
		if ( ! gtw_check_is_array( $cart_contents ) ) {
			return $count ;
		}

		foreach ( $cart_contents as $key => $value ) {
			if ( ! isset( $value[ 'gtw_gift_wrapper' ][ 'mode' ] ) ) {
				continue ;
			}

			if ( 'order' != $value[ 'gtw_gift_wrapper' ][ 'mode' ] ) {
				continue ;
			}

			$count++ ;
		}

		return $count ;
	}

}

if ( ! function_exists( 'gtw_product_gift_wrapper_in_cart' ) ) {

	/**
	 * Get product gift wrapper in cart.
	 *
	 * @return bool/String
	 */
	function gtw_product_gift_wrapper_in_cart() {
		// Return if a cart object is not initialized.
		if ( ! is_object( WC()->cart ) ) {
			return false ;
		}

		// Return false if the cart is empty.
		$cart_contents = WC()->cart->get_cart() ;
		if ( ! gtw_check_is_array( $cart_contents ) ) {
			return false ;
		}

		foreach ( $cart_contents as $key => $value ) {
			if ( ! isset( $value[ 'gtw_gift_wrapper' ][ 'mode' ] ) ) {
				continue ;
			}

			if ( 'product' != $value[ 'gtw_gift_wrapper' ][ 'mode' ] ) {
				continue ;
			}

			return $key ;
		}

		return false ;
	}

}

if ( ! function_exists( 'gtw_order_gift_wrapper_in_cart' ) ) {

	/**
	 * Get order gift wrapper in cart.
	 *
	 * @return bool/String
	 */
	function gtw_order_gift_wrapper_in_cart() {
		// Return if a cart object is not initialized.
		if ( ! is_object( WC()->cart ) ) {
			return false ;
		}

		// Return false if the cart is empty.
		$cart_contents = WC()->cart->get_cart() ;
		if ( ! gtw_check_is_array( $cart_contents ) ) {
			return false ;
		}

		foreach ( $cart_contents as $key => $value ) {
			if ( ! isset( $value[ 'gtw_gift_wrapper' ][ 'mode' ] ) ) {
				continue ;
			}

			if ( 'order' != $value[ 'gtw_gift_wrapper' ][ 'mode' ] ) {
				continue ;
			}

			return $key ;
		}

		return false ;
	}

}

if ( ! function_exists( 'gtw_get_order_gift_wrapper_exclude_products' ) ) {

	/**
	 * Get the order gift wrapper exclude products.
	 *
	 * @return array
	 */
	function gtw_get_order_gift_wrapper_exclude_products() {
		$order_exclude_products = array() ;
		// Return if a cart object is not initialized.
		if ( ! is_object( WC()->cart ) ) {
			return $order_exclude_products ;
		}

		// Return empty array if the cart is empty.
		$cart_contents = WC()->cart->get_cart() ;
		if ( ! gtw_check_is_array( $cart_contents ) ) {
			return $order_exclude_products ;
		}

		foreach ( $cart_contents as $cart_content ) {
			// Don't consider if the product is a gift wrapper.
			if ( isset( $cart_content[ 'gtw_gift_wrapper' ] ) ) {
				continue ;
			}

			// Don't consider the product for order gift wrapper.
			if ( ! gtw_is_cart_content_order_gift_wrapper_restricted( $cart_content ) ) {
				continue ;
			}

			$order_exclude_products[] = ! empty( $cart_content[ 'variation_id' ] ) ? $cart_content[ 'variation_id' ] : $cart_content[ 'product_id' ] ;
		}

		return $order_exclude_products ;
	}

}

if ( ! function_exists( 'gtw_is_cart_content_order_gift_wrapper_restricted' ) ) {

	/**
	 * Is the cart content order gift wrapper restricted?.
	 *
	 * @param array $cart_content
	 * @since 4.3
	 * 
	 * @return bool
	 */
	function gtw_is_cart_content_order_gift_wrapper_restricted( $cart_content ) {
		// Return true if the product is restricted for order gift wrapper.
		if ( gtw_is_product_restricted( $cart_content[ 'product_id' ] , $cart_content[ 'variation_id' ] ) ) {
			return true ;
		}

		// Return true if the product is virtual.
		if ( 'yes' === get_option( 'gtw_settings_exclude_virtual_products' ) && $cart_content[ 'data' ]->is_virtual() ) {
			return true ;
		}

		// Return true if the current user is restricted.
		if ( gtw_is_user_restricted() ) {
			return true ;
		}

		return false ;
	}

}

if ( ! function_exists( 'gtw_is_product_gift_wrapper_restricted' ) ) {

	/**
	 * Is the product gift wrapper restricted?.
	 *
	 * @param object $product
	 * @since 4.6
	 * 
	 * @return bool
	 */
	function gtw_is_product_gift_wrapper_restricted( $product ) {
		if ( ! is_object( $product ) ) {
			return false ;
		}

		$product_id   = $product->get_parent_id() ? $product->get_parent_id() : $product->get_id() ;
		$variation_id = $product->get_parent_id() ? $product_id : false ;
		// Return true if the product is restricted for product gift wrapper.
		if ( gtw_is_product_restricted( $product_id , $variation_id ) ) {
			return true ;
		}

		// Return true if the product is virtual.
		if ( 'yes' === get_option( 'gtw_settings_exclude_virtual_products' ) && $product->is_virtual() ) {
			return true ;
		}

		// Return true if the current user is restricted.
		if ( gtw_is_user_restricted() ) {
			return true ;
		}

		return false ;
	}

}

if ( ! function_exists( 'gtw_is_product_restricted' ) ) {

	/**
	 * Is the product restricted?.
	 *
	 * @param int $product_id
	 * @param int/string $variation_id
	 * @since 4.6
	 * 
	 * @return bool
	 */
	function gtw_is_product_restricted( $product_id, $variation_id = '' ) {
		$return              = false ;
		$product_restriction = get_option( 'gtw_settings_gift_wrapping_product_restriction' ) ;
		// Return true if the option is all products.
		if ( '1' == $product_restriction ) {
			return $return ;
		}

		switch ( $product_restriction ) {
			case '2':
				$include_products = get_option( 'gtw_settings_gift_wrapping_include_product' ) ;
				// Included products.
				if ( ! in_array( $product_id , $include_products ) && ! in_array( $variation_id , $include_products ) ) {
					$return = true ;
				}
				break ;
			case '3':
				$exclude_products = get_option( 'gtw_settings_gift_wrapping_exclude_product' ) ;
				// Excluded products.
				if ( in_array( $product_id , $exclude_products ) || in_array( $variation_id , $exclude_products ) ) {
					$return = true ;
				}
				break ;
			case '4':
				// All Categories.
				$product_categories = get_the_terms( $product_id , 'product_cat' ) ;
				if ( ! gtw_check_is_array( $product_categories ) ) {
					$return = true ;
				}
				break ;
			case '5':
				$return             = true ;
				// Included categories.
				$product_categories = get_the_terms( $product_id , 'product_cat' ) ;
				if ( gtw_check_is_array( $product_categories ) ) {
					$include_categories = get_option( 'gtw_settings_gift_wrapping_include_categories' ) ;
					foreach ( $product_categories as $product_category ) {
						if ( in_array( $product_category->term_id , $include_categories ) ) {
							$return = false ;
						}
					}
				}

				break ;
			case '6':
				// Excluded categories.
				$product_categories = get_the_terms( $product_id , 'product_cat' ) ;
				if ( gtw_check_is_array( $product_categories ) ) {
					$exclude_categories = get_option( 'gtw_settings_gift_wrapping_exclude_categories' ) ;
					foreach ( $product_categories as $product_category ) {
						if ( in_array( $product_category->term_id , $exclude_categories ) ) {
							$return = true ;
						}
					}
				}

				break ;
		}

		return $return ;
	}

}

if ( ! function_exists( 'gtw_is_user_restricted' ) ) {

	/**
	 * Is the user restricted?.
	 *
	 * @since 4.7
	 * 
	 * @return bool
	 */
	function gtw_is_user_restricted() {
		$current_user = wp_get_current_user() ;
		// Return true if the guest user is restricted.
		if ( 'yes' === get_option( 'gtw_settings_guest_user_gift_wrapping_disabled' ) && ! $current_user->exists() ) {
			return true ;
		}

		$return           = false ;
		$user_restriction = get_option( 'gtw_settings_gift_wrapping_user_restriction' ) ;
		switch ( $user_restriction ) {
			// Included users.
			case '2':
				$included_users = get_option( 'gtw_settings_gift_wrapping_included_users' ) ;
				if ( ! in_array( $current_user->ID , $included_users ) ) {
					$return = true ;
				}
				break ;
			// Excluded users.
			case '3':
				$excluded_users = get_option( 'gtw_settings_gift_wrapping_excluded_users' ) ;
				if ( in_array( $current_user->ID , $excluded_users ) ) {
					$return = true ;
				}
				break ;
			// Included user roles.
			case '4':
				$included_user_roles = get_option( 'gtw_settings_gift_wrapping_included_user_roles' ) ;
				if ( ! gtw_check_is_array( $current_user->roles ) || ! array_intersect( $current_user->roles , $included_user_roles ) ) {
					$return = true ;
				}
				break ;
			// Excluded user roles.
			case '5':
				$excluded_user_roles = get_option( 'gtw_settings_gift_wrapping_excluded_user_roles' ) ;
				if ( gtw_check_is_array( $current_user->roles ) && array_intersect( $current_user->roles , $excluded_user_roles ) ) {
					$return = true ;
				}
				break ;
		}

		return $return ;
	}

}

if ( ! function_exists( 'gtw_get_order_gift_wrapper_products' ) ) {

	/**
	 * Get the order gift wrapper products.
	 * 
	 * @param string $return_with
	 * 
	 * @return array
	 */
	function gtw_get_order_gift_wrapper_products( $return_with = 'name' ) {
		$order_products = array() ;
		// Return if a cart object is not initialized.
		if ( ! is_object( WC()->cart ) ) {
			return $order_products ;
		}

		// Return empty array if the cart is empty.
		$cart_contents = WC()->cart->get_cart() ;
		if ( ! gtw_check_is_array( $cart_contents ) ) {
			return $order_products ;
		}

		foreach ( $cart_contents as $cart_content ) {
			// Don't consider if the product is a gift wrapper.
			if ( isset( $cart_content[ 'gtw_gift_wrapper' ] ) ) {
				continue ;
			}

			// Consider the product for order gift wrapper.
			if ( gtw_is_cart_content_order_gift_wrapper_restricted( $cart_content ) ) {
				continue ;
			}

			$product_id       = ! empty( $cart_content[ 'variation_id' ] ) ? $cart_content[ 'variation_id' ] : $cart_content[ 'product_id' ] ;
			$order_products[] = ( 'id' == $return_with ) ? $product_id : $cart_content[ 'data' ]->get_name() ;
		}

		return $order_products ;
	}

}

if ( ! function_exists( 'gtw_order_gift_wrapper_cart_qty' ) ) {

	/**
	 * Order gift wrapper cart quantity.
	 *
	 * @return string
	 */
	function gtw_order_gift_wrapper_cart_qty() {
		if ( '2' != get_option( 'gtw_settings_multiply_gift_wrapper_price' ) ) {
			return 1 ;
		}

		$cart_qty = gtw_get_valid_order_gift_wrapper_cart_qty() ;

		/**
		 * This hook is used to alter order gift wrapper cart quantity.
		 * 
		 * @since 1.0
		 */
		return apply_filters( 'gtw_order_gift_wrapper_cart_qty' , $cart_qty ) ;
	}

}

if ( ! function_exists( 'gtw_get_valid_order_gift_wrapper_cart_qty' ) ) {

	/**
	 * Get valid order gift wrapper cart quantity.
	 *
	 * @since 5.3.0
	 * @return string
	 */
	function gtw_get_valid_order_gift_wrapper_cart_qty() {
		$cart_qty = 0 ;
		// Return if a cart object is not initialized.
		if ( ! is_object( WC()->cart ) ) {
			return $cart_qty ;
		}

		// Return false if the cart is empty.
		$cart_contents = WC()->cart->get_cart() ;
		if ( ! gtw_check_is_array( $cart_contents ) ) {
			return $cart_qty ;
		}

		$exclude_products = gtw_get_order_gift_wrapper_exclude_products() ;

		foreach ( $cart_contents as $cart_content ) {

			// Don't consider if the product is a gift wrapper and order gift wrapper.
			if ( isset( $cart_content[ 'gtw_gift_wrapper' ] ) && 'order' ===$cart_content[ 'gtw_gift_wrapper' ][ 'mode' ] ) {
				continue ;
			}

			$product_id = ! empty( $cart_content[ 'variation_id' ] ) ? $cart_content[ 'variation_id' ] : $cart_content[ 'product_id' ] ;

			// Don't consider if the product is excluded.
			if ( in_array( $product_id , $exclude_products ) ) {
				continue ;
			}

			$cart_qty += $cart_content[ 'quantity' ] ;
		}
		/**
		 * This hook is used to alter valid order gift wrapper cart quantity.
		 * 
		 * @since 1.0
		 */
		return apply_filters( 'gtw_get_valid_order_gift_wrapper_cart_qty' , $cart_qty , $exclude_products ) ;
	}

}

if ( ! function_exists( 'gtw_get_gift_wrapper_popup_description' ) ) {

	/**
	 * Get the gift wrapper popup description.
	 *
	 * @since 5.2.0
	 * @return string
	 */
	function gtw_get_gift_wrapper_popup_description() {
		return get_option( 'gtw_settings_gift_wrapper_popup_description' ) ;
	}

}

if ( ! function_exists( 'gtw_get_select_design_button_localization' ) ) {

	/**
	 * Get select gift wrapper cart item label.
	 *
	 * @since 5.3.0
	 * @return string
	 */
	function gtw_get_select_design_button_localization() {
		return get_option( 'gtw_settings_select_design_button_localization' ) ;
	}

}

if ( ! function_exists( 'gtw_get_update_design_button_localization' ) ) {

	/**
	 * Get update gift wrapper cart item label.
	 *
	 * @since 5.2.0
	 * @return string
	 */
	function gtw_get_update_design_button_localization() {
		return get_option( 'gtw_settings_update_design_button_localization' ) ;
	}

}    
